#include <iostream>

// подключаем основной хедер библиотеки BuDDy
#include <bdd.h>

void empty_gbc_handler(int pre, bddGbcStat *stat) {}

int main(int argc, char ** argv) {
  // случайные числа на то, какие должны быть общий размер массива, хранящего все вершины bdd, и размера кэша вершин
  const size_t node_num = 20000000;
  const size_t cache_size = 2000000;
  bdd_init(node_num, cache_size);
  
  // иногда сборщик мусора может что-то сообщать, и если хотим, чтобы он был тихим и не мешал, то задаём тихий обработчик сообщений от сборщика
  bdd_gbc_hook(empty_gbc_handler);
  
  // явно говорим, какие хотим переменные, тут - x0 ... x4
  bdd_setvarnum(5);
  
  // a = x0, b = x1
  bdd a = bdd_ithvar(0);
  bdd b = bdd_ithvar(1);
  
  // c = x0 & x1
  bdd c = a & b;
  std::cout << (c == bddfalse);
  
  // d = x0 & x1 & !x1
  bdd d = c & !b;
  std::cout << (d == bddfalse);
  
  // e = x0 & x1 | x0 & x1 & !x1
  bdd e = c | d;
  std::cout << (e == bddfalse);
  
  // заводим переменную под множество пар переменных для переименования значений
  bddPair * p;
  
  // инициализация: p - пустое множество пар переменных
  p = bdd_newpair();
  
  // p = {[x0 -> x1]}
  bdd_setpair(p, 0, 1);
  
  // p = {[x0 -> x1], [x1 -> x3], [x3 -> x4]}
  const int size = 2;
  int from[size], to[size];
  from[0] = 1;
  from[1] = 2;
  to[0] = 3;
  to[1] = 4;
  bdd_setpairs(p, from, to, size);
  
  // f = (x0 & x1){[x0 -> x1], [x1 -> x3], [x3 -> x4]} = x1 & x3
  bdd f = bdd_replace(c, p);
  
  // g = x3 & x1; g == f
  bdd g = bdd_ithvar(3) & bdd_ithvar(1);
  std::cout << (f == g);
  
  // завершение работы с множеством пар переменных
  bdd_freepair(p);
  
  std::cout << std::endl;
  
  // завершаем работу с bdd - удаляем массив всех вершин и кэш вершин
  bdd_done();
  
  return 0;
}
